#
# Copyright (c) 1995 The Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#	This product includes software developed by the Computer Systems
#	Engineering Group at Lawrence Berkeley Laboratory.
# 4. Neither the name of the University nor of the Laboratory may be used
#    to endorse or promote products derived from this software without
#    specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# @(#) $Header: /usr/src/mash/repository/vint/ns-2/tcl/test/test-suite-schedule.tcl,v 1.3 1998/08/14 20:14:24 tomh Exp $
#
# To view a list of available tests to run with this script:
# ns test-suite-schedule.tcl
#

set quiet false

source misc.tcl
source topologies.tcl


TestSuite instproc finish file {
	global quiet
        exec ../../bin/getrc -s 2 -d 3 all.tr | \
          ../../bin/raw2xg -s 0.01 -m 90 -t $file > temp.rands
	if {$quiet == "false"} {
		exec xgraph -bb -tk -nl -m -x time -y packets temp.rands &
	}
        ## now use default graphing tool to make a data file
	## if so desired
        exit 0
}

TestSuite instproc printtimers { tcp time} {
	global quiet
	if {$quiet == "false"} {
        	puts "time: $time sRTT(in ticks): [$tcp set srtt_]/8 RTTvar(in ticks): [$tcp set rttvar_]/4 backoff: [$tcp set backoff_]"
	}
}

TestSuite instproc printtimersAll { tcp time interval } {
        $self instvar dump_inst_ ns_
        if ![info exists dump_inst_($tcp)] {
                set dump_inst_($tcp) 1
                $ns_ at $time "$self printtimersAll $tcp $time $interval"
                return
        }
	set newTime [expr [$ns_ now] + $interval]
	$ns_ at $time "$self printtimers $tcp $time"
        $ns_ at $newTime "$self printtimersAll $tcp $newTime $interval"
}

TestSuite instproc runDetailed {} {
	global quiet
	$self instvar ns_ node_ testName_

	# Set up TCP connection
	set tcp1 [$ns_ create-connection TCP $node_(s1) TCPSink $node_(k1) 0]
	$tcp1 set window_ 20
	set ftp1 [$tcp1 attach-app FTP]
	$ns_ at 0.1 "$ftp1 start"
	

	# Set up TCP connection
	set tcp2 [$ns_ create-connection TCP $node_(s2) TCPSink $node_(k1) 1]
	$tcp2 set window_ 20
	set ftp2 [$tcp2 attach-app FTP]
	$ns_ at 0.4 "$ftp2 start"

        $self tcpDump $tcp1 5.0
        $self tcpDump $tcp2 5.0

        $self traceQueues $node_(r1) [$self openTrace 10.0 $testName_]
        $ns_ run
}

#
# Links1 uses 8Mb, 5ms feeders, and a 800Kb 20ms bottleneck.
# Queue-limit on bottleneck is 25 packets.
#
Class Topology/netSFQ -superclass NodeTopology/4nodes

Topology/netSFQ instproc init ns {
	$self next $ns
	$self instvar node_
        $ns duplex-link $node_(s1) $node_(r1) 8Mb 2ms DropTail
        $ns duplex-link $node_(s2) $node_(r1) 8Mb 20ms DropTail
        $ns duplex-link $node_(r1) $node_(k1) 800Kb 2ms SFQ
        $ns queue-limit $node_(r1) $node_(k1) 25
        $ns queue-limit $node_(k1) $node_(r1) 25 
	if {[$class info instprocs config] != ""} {
	 $self config $ns
 	}

}

Class Topology/netDRR -superclass NodeTopology/4nodes
Topology/netDRR instproc init ns {
	$self next $ns
	$self instvar node_
        $ns duplex-link $node_(s1) $node_(r1) 8Mb 2ms DropTail
        $ns duplex-link $node_(s2) $node_(r1) 8Mb 10ms DropTail
        $ns duplex-link $node_(r1) $node_(k1) 800Kb 2ms DRR
        $ns queue-limit $node_(r1) $node_(k1) 25
        $ns queue-limit $node_(k1) $node_(r1) 25 
        if {[$class info instprocs config] != ""} {
         $self config $ns
        }
}

Class Topology/netRED -superclass NodeTopology/4nodes
Topology/netRED instproc init ns {
	$self next $ns
	$self instvar node_
        $ns duplex-link $node_(s1) $node_(r1) 8Mb 2ms DropTail
        $ns duplex-link $node_(s2) $node_(r1) 8Mb 10ms DropTail
        $ns duplex-link $node_(r1) $node_(k1) 800Kb 2ms RED 
        $ns queue-limit $node_(r1) $node_(k1) 25
        $ns queue-limit $node_(k1) $node_(r1) 25 
        if {[$class info instprocs config] != ""} {
         $self config $ns
        }
}

Class Topology/netDT -superclass NodeTopology/4nodes
Topology/netDT instproc init ns {
	$self next $ns
	$self instvar node_
        $ns duplex-link $node_(s1) $node_(r1) 8Mb 2ms DropTail
        $ns duplex-link $node_(s2) $node_(r1) 8Mb 10ms DropTail
        $ns duplex-link $node_(r1) $node_(k1) 800Kb 2ms DropTail
        $ns queue-limit $node_(r1) $node_(k1) 25
        $ns queue-limit $node_(k1) $node_(r1) 25 
        if {[$class info instprocs config] != ""} {
         $self config $ns
        }
}

#######################################################

Class Test/fifo-droptail -superclass TestSuite
Test/fifo-droptail instproc init topo {
        $self instvar net_ defNet_ test_
        set net_        $topo
        set defNet_     netDT
        set test_       fifo-droptail
        $self next
}

Test/fifo-droptail instproc run {} {
	Agent/TCP set overhead_ 0.01
	$self runDetailed
}

Class Test/fifo-red -superclass TestSuite
Test/fifo-red instproc init topo {
        $self instvar net_ defNet_ test_
        set net_        $topo
        set defNet_     netRED
        set test_       fifo-red
        $self next
}

Test/fifo-red instproc run {} {
	$self runDetailed
}

Class Test/sfq -superclass TestSuite
Test/sfq instproc init topo {
        $self instvar net_ defNet_ test_
        set net_        $topo
        set defNet_     netSFQ
        set test_       sfq
        $self next
}

Test/sfq instproc run {} {
	$self runDetailed
}

Class Test/drr -superclass TestSuite
Test/drr instproc init topo {
        $self instvar net_ defNet_ test_
        set net_        $topo
        set defNet_     netDRR
        set test_       drr
        $self next
}

Test/drr instproc run {} {
	$self runDetailed
}


TestSuite runTest

#######################################################
